package com.compomics.util.experiment.io.identifications.idfilereaders;

import com.compomics.util.experiment.identification.matches.SpectrumMatch;
import com.compomics.util.experiment.massspectrometry.SpectrumFactory;
import java.util.ArrayList;
import java.util.HashMap;

/**
 * IdfileReader for identification files in the SQT format written according to
 * https://noble.gs.washington.edu/proj/crux/sqt-format.html.
 *
 * @author Marc
 */
public class SqtIdfileReader {

    /**
     * The name of the software used to create the SQT file
     */
    private String sqtGenerator;

    /**
     * The version number of the SQTGenerator software
     */
    private String sqtGeneratorVersion;

    /**
     * Path to sequence database used to generate the SQT file
     */
    private String databasePath;

    /**
     * Were average or mono-istopic residue masses used to predict the fragment
     * ion mass
     */
    private String fragmentMasses;

    /**
     * Were average or mono-istopic residue masses used to predict the precursor
     * ion mass
     */
    private String precursorMasses;

    /**
     * Time when SQT file was created
     */
    private String startTime;

    /**
     * Non-standard amino-acid masses used in identification (repeat this record
     * if there are multiple non-standard masses)
     */
    private String staticMod;

    /**
     * List of dynamic modifications used in identification
     */
    private String dynamicMod;

    /**
     * Remarks
     */
    private ArrayList<String> comments;

    /**
     * Number of aminio acids in sequence database
     */
    private String dbSeqLength;

    /**
     * Number of proteins in sequence database
     */
    private String dbLocusCount;

    /**
     * MD5 checksum of sequence database
     */
    private String dbMD5Sum;

    /**
     * Name of field use to sort spectra
     */
    private String sortedBy;

    /**
     * Field names begining with Alg- are algorithm specific
     */
    private HashMap<String, ArrayList<String>> alg;

    /**
     * Other field names are allowed, but may not contain white-space
     */
    private HashMap<String, ArrayList<String>> otherHeader;
    /**
     * The spectrum factory.
     */
    private SpectrumFactory spectrumFactory = SpectrumFactory.getInstance();

    private String currentSpectrumKey = null;

    private ArrayList<SpectrumMatch> spectrumMatches;

    private SpectrumMatch currentSpectrumMatch;
    
    private Integer currentCharge = null;

    private void parseSpectrumLine(String line) {
        if (currentSpectrumMatch != null) {
            spectrumMatches.add(currentSpectrumMatch);
        }
        String[] split = line.split("\t");
    }

    /**
     * Parses a header line and puts the information in the relevant attribute.
     *
     * @param line a header line
     */
    private void parseHeaderLine(String line) {
        int whiteSpaceIndex = line.indexOf("\t");
        String subline = line.substring(whiteSpaceIndex);
        subline = subline.trim();
        if (subline.startsWith("SQTGenerator")) {
            whiteSpaceIndex = line.indexOf("\t");
            String value = subline.substring(whiteSpaceIndex);
            value = value.trim();
            sqtGenerator = value;
        } else if (subline.startsWith("SQTGeneratorVersion")) {
            whiteSpaceIndex = line.indexOf("\t");
            String value = subline.substring(whiteSpaceIndex);
            value = value.trim();
            sqtGeneratorVersion = value;
        } else if (subline.startsWith("Database")) {
            whiteSpaceIndex = line.indexOf("\t");
            String value = subline.substring(whiteSpaceIndex);
            value = value.trim();
            databasePath = value;
        } else if (subline.startsWith("FragmentMasses")) {
            whiteSpaceIndex = line.indexOf("\t");
            String value = subline.substring(whiteSpaceIndex);
            value = value.trim();
            fragmentMasses = value;
        } else if (subline.startsWith("PrecursorMasses")) {
            whiteSpaceIndex = line.indexOf("\t");
            String value = subline.substring(whiteSpaceIndex);
            value = value.trim();
            precursorMasses = value;
        } else if (subline.startsWith("StartTime")) {
            whiteSpaceIndex = line.indexOf("\t");
            String value = subline.substring(whiteSpaceIndex);
            value = value.trim();
            startTime = value;
        } else if (subline.startsWith("StaticMod")) {
            whiteSpaceIndex = line.indexOf("\t");
            String value = subline.substring(whiteSpaceIndex);
            value = value.trim();
            staticMod = value;
        } else if (subline.startsWith("DynamicMod")) {
            whiteSpaceIndex = line.indexOf("\t");
            String value = subline.substring(whiteSpaceIndex);
            value = value.trim();
            dynamicMod = value;
        } else if (subline.startsWith("Comment")) {
            whiteSpaceIndex = line.indexOf("\t");
            String value = subline.substring(whiteSpaceIndex);
            value = value.trim();
            String comment = value;
            comments.add(comment);
        } else if (subline.startsWith("DBSeqLength")) {
            whiteSpaceIndex = line.indexOf("\t");
            String value = subline.substring(whiteSpaceIndex);
            value = value.trim();
            dbSeqLength = value;
        } else if (subline.startsWith("DBLocusCount")) {
            whiteSpaceIndex = line.indexOf("\t");
            String value = subline.substring(whiteSpaceIndex);
            value = value.trim();
            dbLocusCount = value;
        } else if (subline.startsWith("DBMD5Sum")) {
            whiteSpaceIndex = line.indexOf("\t");
            String value = subline.substring(whiteSpaceIndex);
            value = value.trim();
            dbMD5Sum = value;
        } else if (subline.startsWith("SortedBy")) {
            whiteSpaceIndex = line.indexOf("\t");
            String value = subline.substring(whiteSpaceIndex);
            value = value.trim();
            sortedBy = value;
        } else if (subline.startsWith("Alg")) {
            whiteSpaceIndex = line.indexOf("\t");
            String key = subline.substring(4, whiteSpaceIndex);
            String value = subline.substring(whiteSpaceIndex);
            value = value.trim();
            ArrayList<String> values = alg.get(key);
            if (values == null) {
                values = new ArrayList<String>();
                alg.put(key, values);
            }
            values.add(value);
        } else {

        }
    }
}
